@@ -36,6 +36,74 @@ logLikDccGarch <- function(mY, omega=rep(0.03, ncol(mY)), alpha=rep(0.03, ncol(m
   return(out2)
 }
 
+updateBayesDccGarch <- function(x){
+  mY = x$control$data
+  n = nrow(mY)
+  k = ncol(mY)
+  n_mcmc = nrow(x$MC)
+  
+  if(k==1){
+	x$MC = cbind(x$MC, 0.1, 0.1) ## include a,b parameters	
+  }
+  
+  if(n_mcmc > 2000){
+    ss =  round(seq.int(from=1, to=n_mcmc, length.out=2000)) #sample.int(n=n_mcmc, size = 2000, replace = FALSE)
+  }else{
+	ss = 1:n_mcmc
+  }
+  
+  out = .C("loglike_matrix_R", 
+    vY = as.numeric(t(mY)), 
+    vn = as.integer(nrow(mY)), 
+    vk = as.integer(ncol(mY)), 
+    vmcmc = as.numeric(t(x$MC[ss,])), 
+    n_mcmc = as.integer(length(ss)), #as.integer(n_mcmc), 
+    verrorDist = as.integer(x$control$errorDist), 
+    meanLogLike = numeric(1), 
+    vmMeanH = numeric(nrow(mY)*ncol(mY)*ncol(mY))
+  )
+  
+  if(k==1){
+	x$MC = as.mcmc( x$MC[,1:(4*k+1)] ) ## exclude a,b parameters	
+  }
+	 
+  D_mean = -2 * out$meanLogLike
+  par_est = apply(X=x$MC, MARGIN=2, FUN=mean)
+  tail_est = par_est[1]; gamma_est = par_est[2+ 4*(0:(k-1))]; omega_est = par_est[3+ 4*(0:(k-1))];
+  alpha_est = par_est[4+ 4*(0:(k-1))]; beta_est = par_est[5+ 4*(0:(k-1))];
+  if(k>1){ a_est = par_est[2+4*k]; b_est = par_est[3+4*k];  }else{ a_est = 0.1; b_est = 0.1;}
+	
+  D_at_mean = -2*logLikDccGarch(mY, omega_est, alpha_est, beta_est, a_est, b_est, gamma_est, tail_est, x$control$errorDist)$value
+
+  if( k == 1 ){ npar = 5 }else{ npar = 4*k +3 }
+  
+  if(x$control$errorDist==1){ 
+    EAIC = D_mean + 2*(npar-1)
+	EBIC = D_mean + (npar-1)*log(n)	
+  }else{ 
+	EAIC = D_mean + 2*npar
+	EBIC = D_mean + npar*log(n)	
+  }
+  DIC = D_at_mean + 2*(D_mean - D_at_mean)
+
+  IC = matrix(c(EAIC, EBIC, DIC), nrow=3, ncol=1)
+  rownames(IC) = c("EAIC", "EBIC", "DIC")	
+  colnames(IC) = c("Estimative")
+  x$IC = IC
+  
+  Hname = rep("NA", k*k)
+  for(i in 1:k){ for(j in 1:k){ Hname[(i-1)*k + j] = paste("H_",i,",",j,sep="") } }
+  
+  #Hname = colnames(x$H) 
+  x$H = matrix(out$vmMeanH, ncol=ncol(mY)^2, byrow=T)
+  colnames(x$H) = Hname 
+  
+  #x$logLike = out$meanLogLike
+  
+  return(x)
+}
+
+
 MH_oneDimension <- function(phi_ini, k, n_sim, sd_phi_sim){
 
   if( k == 1 )
@@ -147,7 +215,7 @@ bayesDccGarch <- function(mY, nSim=10000, tail_ini=8, omega_ini=rep(0.03, ncol(m
 	if( control$simAlg == 3 ){ 
 		writeLines("Maximizing the log-posterior density function.")
 		if(k==1) phi_ini = phi_ini[1:5]
-		opt = optim(par = phi_ini, fn = logPosterior_phi, hessian=TRUE, control = list(fnscale=-1))
+		opt = optim(par = phi_ini, fn = logPosterior_phi, hessian=TRUE, control = list(fnscale=-abs(Lout$value))) #optim(par = phi_ini, fn = logPosterior_phi, hessian=TRUE, control = list(fnscale=-1))
 		if(k==1) phi_ini = c(phi_ini,-2,1)
 		
 		if(opt$convergence==0 || opt$convergence==1){
@@ -156,14 +224,18 @@ bayesDccGarch <- function(mY, nSim=10000, tail_ini=8, omega_ini=rep(0.03, ncol(m
 			if(k==1) phi_ini = c(phi_ini,-2,1)
 			
 			control$cholCov = try( t( chol( solve( -opt$hessian ) ) ) , silent = TRUE )
-			if( class(control$cholCov) == "try-error"){
-				control$cholCov = try( t( chol( solve( -hessian(func=logPosterior_phi, x=opt$par) ) ) ) , silent = TRUE )
-			}
+			# if( class(control$cholCov) == "try-error"){
+				# nearPD_Hessian = nearPD( -opt$hessian ) 
+				
+				# if( nearPD_Hessian$converged ){
+				  # control$cholCov = try( t( chol( solve( nearPD_Hessian$mat ) ) ) , silent = TRUE )
+				# }
+			# }
 			
 			if( class(control$cholCov) != "try-error"){
 				option = 1
 			}else{
-				writeLines("The approximated hessian matrix is not positive definitely.")   
+				writeLines("One approximation for covariance matrix of parameters cannot be directly computed through the hessian matrix.")   
 				option = 2
 			}
 		}else{
@@ -318,26 +390,29 @@ bayesDccGarch <- function(mY, nSim=10000, tail_ini=8, omega_ini=rep(0.03, ncol(m
     out = list()
 	out$control = control
     out$MC = as.mcmc(MC)
-	out$H = mMeanH
+	out$H = mMeanH 
 	out$IC = IC
-    out$elapsedTime = proc.time() - ptm  
-    return(out)
+    out$elapsedTime = proc.time() - ptm
+	# out$logLike = logLike_mean
+	
+    return(structure(out,class="bayesDccGarch"))
 }
 
 
-increaseSim <- function(outList, nSim=10000)
+increaseSim <- function(x, nSim=10000)
 {
-  control = outList$control
+  if(class(x) != "bayesDccGarch"){ stop("Error: argument x is not one element of 'bayesDccGarch' class") }	
+  control = x$control
   k = ncol(control$data)
-  nMC = nrow(outList$MC)
-  tail_ini = outList$MC[nMC,1]
-  gamma_ini = outList$MC[nMC,2 + 4*(0:(k-1))]
-  omega_ini = outList$MC[nMC,3 + 4*(0:(k-1))]
-  alpha_ini = outList$MC[nMC,4 + 4*(0:(k-1))]
-  beta_ini =  outList$MC[nMC,5 + 4*(0:(k-1))]
+  nMC = nrow(x$MC)
+  tail_ini = x$MC[nMC,1]
+  gamma_ini = x$MC[nMC,2 + 4*(0:(k-1))]
+  omega_ini = x$MC[nMC,3 + 4*(0:(k-1))]
+  alpha_ini = x$MC[nMC,4 + 4*(0:(k-1))]
+  beta_ini =  x$MC[nMC,5 + 4*(0:(k-1))]
   if(k > 1){
-    a_ini = outList$MC[nMC, 2 + 4*k]
-    b_ini = outList$MC[nMC, 3 + 4*k]
+    a_ini = x$MC[nMC, 2 + 4*k]
+    b_ini = x$MC[nMC, 3 + 4*k]
   }else{
     a_ini = 0.03
 	b_ini = 0.8
@@ -347,12 +422,24 @@ increaseSim <- function(outList, nSim=10000)
   
   outNewSim = bayesDccGarch(control$data, nSim, tail_ini, omega_ini, alpha_ini, beta_ini, a_ini, b_ini, gamma_ini, control$errorDist, control)
   
-  outList$H = outNewSim$H #(nMC*outList$H + nSim*outNewSim$H)/(nMC + nSim)
-  outList$IC = outNewSim$IC
-  outList$MC = as.mcmc( rbind(outList$MC, outNewSim$MC) )
-  outList$elapsedTime = outList$elapsedTime + outNewSim$elapsedTime
+  x$H = (nMC*x$H + nSim*outNewSim$H)/(nMC + nSim)
+  x$IC = (nMC*x$IC + nSim*outNewSim$IC)/(nMC + nSim)
+  x$MC = as.mcmc( rbind(x$MC, outNewSim$MC) )
+  x$elapsedTime = x$elapsedTime + outNewSim$elapsedTime
   
-  return(outList)
+  return(x)
+}
+
+window.bayesDccGarch <- function(x, start = NULL, end = NULL, thin = NULL,...){
+	if(class(x) != "bayesDccGarch"){ stop("Error: argument x is not a element of 'bayesDccGarch' class") }
+	nMC = nrow(x$MC)
+	if(is.null(start)){start = 1}
+	if(is.null(end)){end = nMC}
+	if(is.null(thin)){thin = 1}
+
+	x$MC = window(as.mcmc(x$MC), start = start, end = end, thin = thin,...)
+	
+	return( updateBayesDccGarch(x) )
 }
 
 
@@ -368,7 +455,6 @@ plotVol <- function(mY, vol, ts.names=paste("TS_", 1:ncol(mY), sep=""), colors =
   
   n = nrow(mY)
   k = ncol(mY)
-  k = ncol(mY)
   
   par(mfrow=c(k,1), mar= c(4, 4, 2, 2))
   for(i in 1:k){
@@ -378,3 +464,46 @@ plotVol <- function(mY, vol, ts.names=paste("TS_", 1:ncol(mY), sep=""), colors =
   }
 }
 
+
+summary.bayesDccGarch <- function(object,...){
+	summary_mcmc = summary(as.mcmc(object$MC), ...)
+	return(summary_mcmc)
+}
+
+print.bayesDccGarch <- function(x,...)
+{
+	dist_names = c("Standard Skew Normal", "Standard Skew t-Student", "Standard Skew GED")	
+	
+	cat("Bayesian DCC-GARCH(1,1) model\n")
+	cat( paste("Error Distribution:", dist_names[x$control$errorDist], "\n" ) )
+	
+	print( summary(x, ...) )
+	
+	cat( "3. Information criterions:\n" )
+	
+	print( x$IC )
+	# summary.out = summary(x, ...)
+	
+	# cat("Statistics:\n")
+	# print( summary.out$statistics )
+	# cat("\nQuantiles:\n")
+	# print( summary.out$quantiles )  
+}
+
+plot.bayesDccGarch <- function(x, ts.names=NULL, colors = c("grey","red"), ...){
+  
+  mY = as.matrix( x$control$data )
+  nTs = ncol(mY)
+  vol = as.matrix(x$H[,paste("H_", 1:nTs, ",", 1:nTs, sep="")])
+  if( is.null(ts.names) ){
+	if( is.null(colnames(mY)) ){
+	  ts.names = paste("TS_", 1:nTs, sep="")
+	}else{
+	  ts.names = colnames(mY)
+	}
+  }
+  
+  plotVol(mY, vol, ts.names, colors, ...) 
+}
+
+
